@@ -3,7 +3,7 @@
 #  misaka.py - a "美坂" compatible Shiori module for ninix
 #  Copyright (C) 2002 by Tamito KAJIYAMA
 #  Copyright (C) 2002, 2003 by MATSUMURA Namihiko <nie@counterghost.net>
-#  Copyright (C) 2002-2011 by Shyouzou Sugitani <shy@users.sourceforge.jp>
+#  Copyright (C) 2002-2013 by Shyouzou Sugitani <shy@users.sourceforge.jp>
 #
 #  This program is free software; you can redistribute it and/or modify it
 #  under the terms of the GNU General Public License (version 2) as
@@ -20,7 +20,7 @@
 
 import os
 import re
-import StringIO
+import io
 import sys
 import logging
 import time
@@ -42,40 +42,40 @@ def lexical_error(path=None, position=None):
     if path is None and position is None:
         error = 'lexical error'
     elif path is None:
-        at = 'line %d, column %d' % (position)
-        error = 'lexical error at %s' % at
+        at = 'line {0:d}, column {1:d}'.format(*position)
+        error = 'lexical error at {0}'.format(at)
     elif position is None:
-        error = 'lexical error in %s' % path
+        error = 'lexical error in {0}'.format(path)
     else:
-        at = 'line %d, column %d' % (position)
-        error = 'lexical error at %s in %s' % (at, path)
-    raise MisakaError, error
+        at = 'line {0:d}, column {1:d}'.format(*position)
+        error = 'lexical error at {0} in {1}'.format(at, path)
+    raise MisakaError(error)
 
 def syntax_error(message, path=None, position=None):
     if path is None and position is None:
-        error = 'syntax error (%s)' % message
+        error = 'syntax error ({0})'.format(message)
     elif path is None:
-        at = 'line %d, column %d' % position
-        error = 'syntax error at %s (%s)' % (at, message)
+        at = 'line {0:d}, column {1:d}'.format(*position)
+        error = 'syntax error at {0} ({1})'.format(at, message)
     elif position is None:
-        error = 'syntax error in %s (%s)' % (path, message)
+        error = 'syntax error in {0} ({1})'.format(path, message)
     else:
-        at = 'line %d, column %d' % position
-        error = 'syntax error at %s in %s (%s)' % (at, path, message)
-    raise MisakaError, error
+        at = 'line {0:d}, column {1:d}'.format(*position)
+        error = 'syntax error at {0} in {1} ({2})'.format(at, path, message)
+    raise MisakaError(error)
 
 def list_dict(top_dir):
     buf = []
     try:
         filelist, debug, error = read_misaka_ini(top_dir)
-    except Exception, e:
+    except Exception as e:
         filelist = []
     for filename in filelist:
         buf.append(os.path.join(top_dir, filename))
     return buf
 
-def read_misaka_ini(top_dir, _debug=0):
-    path = os.path.join(top_dir, 'misaka.ini')
+def read_misaka_ini(top_dir):
+    path = os.path.join(top_dir, b'misaka.ini')
     with open(path) as f:
         filelist = []
         debug = 0
@@ -100,7 +100,7 @@ def read_misaka_ini(top_dir, _debug=0):
                         break
                     if not line or line.startswith('//'):
                         continue
-                    filelist.append(get_normalized_path(line, encode=0))
+                    filelist.append(get_normalized_path(line))
             elif line == 'debug,0':
                 debug = 0
             elif line == 'debug,1':
@@ -113,8 +113,8 @@ def read_misaka_ini(top_dir, _debug=0):
                 pass
             elif line == 'propertyhandler,1':
                 pass
-            elif _debug & 4:
-                print "misaka.py: unknown directive '%s'" % line
+            else:
+                logging.debug("misaka.py: unknown directive '{0}'".format(line))
     return filelist, debug, error
 
 ###   LEXER   ###
@@ -154,7 +154,7 @@ class Lexer:
         (TOKEN_OPERATOR,      re.compile(r'[!=]=|[<>]=?|=[<>]|&&|\|\||\+\+|--|[+\-*/]?=|[+\-*/%\^]')),
         (TOKEN_DIRECTIVE,     re.compile(r'#[_A-Za-z][_A-Za-z0-9]*')),
         #(TOKEN_TEXT,          re.compile(r"[!&|]|(\\[,\"]|[\x01\x02#'.0-9:?@A-Z\\_`a-z~]|[\x80-\xff].)+")),
-        (TOKEN_TEXT,          re.compile(ur"(\"[^\"]*\")|[!&|]|(\\[,\"]|[\x01\x02#'.0-9:?@A-Z\\_`a-z~\"]|[\u0080-\uffff])+")),
+        (TOKEN_TEXT,          re.compile(r"(\"[^\"]*\")|[!&|]|(\\[,\"]|[\x01\x02#'.0-9:?@A-Z\\_`a-z~\"]|[" + "\u0080" + "-" + "\uffff" + r"]+)+")),
         ]
     token_names = {
         TOKEN_WHITESPACE:    'whitespace',
@@ -173,9 +173,8 @@ class Lexer:
         TOKEN_TEXT:          'text',
         }
 
-    def __init__(self, charset='Shift_JIS', debug=0):
+    def __init__(self, charset='CP932'):
         self.buffer = []
-        self.debug = debug
         self.charset = charset
 
     def _match(self, data, line, column, path):
@@ -204,7 +203,7 @@ class Lexer:
                     pos = match.end()
                     break
             else:
-                ###print data[pos:pos + 100]
+                ###print(data[pos:pos + 100])
                 lexical_error(path, (line, column))
             if token == TOKEN_NEWLINE:
                 line = line + 1
@@ -216,7 +215,7 @@ class Lexer:
     def read(self, f, path=None):
         line = 1
         column = 0
-        data = unicode(f.read(), self.charset, 'replace')
+        data = f.read()
         line, column = self._match(data, line, column, path)
         self.buffer.append((TOKEN_NEWLINE, '', (line, column)))
         self.path = path
@@ -229,13 +228,13 @@ class Lexer:
             token, lexeme, self.position = self.buffer.pop(0)
         except IndexError:
             syntax_error('unexpected end of file', self.path)
-        ###print token, repr(lexeme)
+        ###print(token, repr(lexeme))
         return token, lexeme
 
     def pop_check(self, expected):
         token, lexeme = self.pop()
         if token != expected:
-            syntax_error(''.join(('exptected ', self.token_names[expected])),
+            syntax_error(''.join(('exptected ', self.token_names[expected], ', but returns ', self.token_names[token])),
                          self.path, self.get_position())
         return lexeme
 
@@ -283,10 +282,9 @@ NODE_POW_EXPR    = 14
 
 class Parser:
 
-    def __init__(self, charset, debug=0):
-        self.lexer = Lexer(charset=charset, debug=debug)
+    def __init__(self, charset):
+        self.lexer = Lexer(charset=charset)
         self.charset = charset
-        self.debug = debug
 
     def read(self, f, path=None):
         self.lexer.read(f, path)
@@ -413,17 +411,15 @@ class Parser:
             else:
                 try:
                     line = self.get_line() # beginning with brace expression
-                except MisakaError, error:
-                    if self.debug & 4:
-                        print error
+                except MisakaError as error:
+                    logging.debug(error)
                     self.lexer.skip_line()
                     return None
         else:
             try:
                 line = self.get_line()
-            except MisakaError, error:
-                if self.debug & 4:
-                    print error
+            except MisakaError as error:
+                logging.debug(error)
                 self.lexer.skip_line()
                 return None
         self.lexer.pop_check(TOKEN_NEWLINE)
@@ -481,7 +477,7 @@ class Parser:
             elif token == TOKEN_OPEN_BRACE:
                 buf.append(self.get_brace_expr())
             else:
-                raise RuntimeError, 'should not reach here'
+                raise RuntimeError('should not reach here')
         # strip whitespace at the beginning and/or end of line
         if buf and self.is_whitespace(buf[0]):
             del buf[0]
@@ -774,73 +770,73 @@ class Parser:
     def dump_node(self, node, depth):
         indent = '  ' * depth
         if node[0] == NODE_TEXT:
-            print ''.join((indent, 'TEXT')), \
-                  ''.join(('"', node[1].encode('utf-8', 'ignore'), '"'))
+            print(''.join((indent, 'TEXT')), \
+                  ''.join(('"', node[1], '"')))
         elif node[0] == NODE_STRING:
-            print ''.join((indent, 'STRING')), \
-                  ''.join(('"', node[1].encode('utf-8', 'ignore'), '"'))
+            print(''.join((indent, 'STRING')), \
+                  ''.join(('"', node[1], '"')))
         elif node[0] == NODE_VARIABLE:
-            print ''.join((indent, 'VARIABLE'))
+            print(''.join((indent, 'VARIABLE')))
             self.dump_list(node[1], depth + 1)
         elif node[0] == NODE_INDEXED:
-            print ''.join((indent, 'INDEXED'))
+            print(''.join((indent, 'INDEXED')))
             self.dump_list(node[1], depth + 1)
-            print ''.join((indent, 'index'))
+            print(''.join((indent, 'index')))
             self.dump_list(node[2], depth + 1)
         elif node[0] == NODE_ASSIGNMENT:
-            print ''.join((indent, 'ASSIGNMENT'))
+            print(''.join((indent, 'ASSIGNMENT')))
             self.dump_list(node[1], depth + 1)
-            print ''.join((indent, 'op')), node[2]
+            print(''.join((indent, 'op')), node[2])
             self.dump_list(node[3], depth + 1)
         elif node[0] == NODE_FUNCTION:
-            print ''.join((indent, 'FUNCTION')), node[1]
+            print(''.join((indent, 'FUNCTION')), node[1])
             for i in range(len(node[2])):
-                print ''.join((indent, 'args[%d]' % i))
+                print(''.join((indent, 'args[{0:d}]'.format(i))))
                 self.dump_list(node[2][i], depth + 1)
         elif node[0] == NODE_IF:
-            print ''.join((indent, 'IF'))
+            print(''.join((indent, 'IF')))
             self.dump_node(node[1], depth + 1)
-            print ''.join((indent, 'then'))
+            print(''.join((indent, 'then')))
             self.dump_list(node[2], depth + 1)
-            print ''.join((indent, 'else'))
+            print(''.join((indent, 'else')))
             self.dump_list(node[3], depth + 1)
         elif node[0] == NODE_CALC:
-            print ''.join((indent, 'CALC'))
+            print(''.join((indent, 'CALC')))
             self.dump_list(node[1], depth + 1)
         elif node[0] == NODE_AND_EXPR:
-            print ''.join((indent, 'AND_EXPR'))
+            print(''.join((indent, 'AND_EXPR')))
             for i in range(1, len(node)):
                 self.dump_node(node[i], depth + 1)
         elif node[0] == NODE_OR_EXPR:
-            print ''.join((indent, 'OR_EXPR'))
+            print(''.join((indent, 'OR_EXPR')))
             for i in range(1, len(node)):
                 self.dump_node(node[i], depth + 1)
         elif node[0] == NODE_COMP_EXPR:
-            print ''.join((indent, 'COMP_EXPR'))
+            print(''.join((indent, 'COMP_EXPR')))
             self.dump_list(node[1], depth + 1)
-            print ''.join((indent, 'op')), node[2]
+            print(''.join((indent, 'op')), node[2])
             self.dump_list(node[3], depth + 1)
         elif node[0] == NODE_ADD_EXPR:
-            print ''.join((indent, 'ADD_EXPR'))
+            print(''.join((indent, 'ADD_EXPR')))
             self.dump_list(node[1], depth + 1)
             for i in range(2, len(node), 2):
-                print ''.join((indent, 'op')), node[i]
+                print(''.join((indent, 'op')), node[i])
                 self.dump_list(node[i + 1], depth + 1)
         elif node[0] == NODE_MUL_EXPR:
-            print ''.join((indent, 'MUL_EXPR'))
+            print(''.join((indent, 'MUL_EXPR')))
             self.dump_list(node[1], depth + 1)
             for i in range(2, len(node), 2):
-                print ''.join((indent, 'op')), node[i]
+                print(''.join((indent, 'op')), node[i])
                 self.dump_list(node[i + 1], depth + 1)
         elif node[0] == NODE_POW_EXPR:
-            print ''.join((indent, 'POW_EXPR'))
+            print(''.join((indent, 'POW_EXPR')))
             self.dump_list(node[1], depth + 1)
             for i in range(2, len(node)):
-                print ''.join((indent, 'op ^'))
+                print(''.join((indent, 'op ^')))
                 self.dump_list(node[i], depth + 1)
         else:
-            print node
-            raise RuntimeError, 'should not reach here'
+            print(node)
+            raise RuntimeError('should not reach here')
 
 # <<< syntax >>>
 # dict       := sp ( common sp )? ( group sp )*
@@ -897,6 +893,7 @@ class Parser:
 # unary_op   := "+" | "-"
 # factor     := OPEN_PAREN sp add_expr sp CLOSE_PAREN | word
 
+
 ###   INTERPRETER   ###
 
 class Group:
@@ -930,7 +927,7 @@ class NonOverlapGroup(Group):
         if not self.list:
             return None
         if not self.indexes:
-            self.indexes = range(len(self.list))
+            self.indexes = list(range(len(self.list)))
         index = self.indexes.pop(random.choice(range(len(self.indexes))))
         return self.list[index]
 
@@ -944,14 +941,14 @@ class SequentialGroup(Group):
         if not self.list:
             return None
         if not self.indexes:
-            self.indexes = range(len(self.list))
+            self.indexes = list(range(len(self.list)))
         index = self.indexes.pop(0)
         return self.list[index]
 
 class Array(Group):
 
     def append(self, s):
-        self.list.append([[NODE_TEXT, unicode(s)]])
+        self.list.append([[NODE_TEXT, str(s)]])
 
     def index(self, s):
         for i in range(len(self.list)):
@@ -981,12 +978,11 @@ TYPE_ARRAY   = 2
 
 class Shiori:
 
-    DBNAME = 'misaka.db'
+    DBNAME = b'misaka.db'
 
-    def __init__(self, dll_name, debug=0):
+    def __init__(self, dll_name):
         self.dll_name = dll_name
-        self.debug = debug
-        self.charset = 'Shift_JIS'
+        self.charset = 'CP932'
 
     def use_saori(self, saori):
         self.saori = saori
@@ -995,7 +991,7 @@ class Shiori:
         result = 0
         if list_dict(top_dir):
             result = 210
-        elif os.path.isfile(os.path.join(top_dir, 'misaka.ini')):
+        elif os.path.isfile(os.path.join(top_dir, b'misaka.ini')):
             result = 200
         return result
 
@@ -1004,7 +1000,7 @@ class Shiori:
             'Shiori: MISAKA compatible module for ninix\n'
             '        Copyright (C) 2002 by Tamito KAJIYAMA\n'
             '        Copyright (C) 2002, 2003 by MATSUMURA Namihiko\n'
-            '        Copyright (C) 2002-2011 by Shyouzou Sugitani')
+            '        Copyright (C) 2002-2013 by Shyouzou Sugitani')
 
     def reset(self):
         self.dict = {}
@@ -1025,15 +1021,12 @@ class Shiori:
         self.saori_library = SaoriLibrary(self.saori, self.misaka_dir)
         self.reset()
         try:
-            filelist, debug, error = read_misaka_ini(self.misaka_dir,
-                                                     self.debug)
+            filelist, debug, error = read_misaka_ini(self.misaka_dir)
         except IOError:
-            if self.debug & 4:
-                print 'cannot read misaka.ini'
+            logging.debug('cannot read misaka.ini')
             return 0
-        except MisakaError, error:
-            if self.debug & 4:
-                print error
+        except MisakaError as error:
+            logging.debug(error)
             return 0
         self.misaka_debug = debug
         self.misaka_error = error
@@ -1045,39 +1038,42 @@ class Shiori:
             for filename in filelist:
                 path = os.path.join(self.misaka_dir, filename)
                 try:
-                    f = open(path)
+                    f = open(path, 'rb')
                 except IOError:
-                    if self.debug & 4:
-                        print 'cannot read', filename
+                    logging.debug('cannot read {0}'.format(filename))
                     continue
                 basename, ext = os.path.splitext(filename)
                 if ext == '.__1':
-                    f = StringIO.StringIO(self.crypt(f.read()))
+                    f = io.StringIO(self.crypt(f.read()))
                 detector.reset()
                 for line in f:
                     detector.feed(line)
                     if detector.done:
                         break
                 detector.close()
+                f.close()
                 if detector.result['confidence'] > 0.98: # XXX
                     self.charset = detector.result['encoding']
+                    if self.charset == 'SHIFT_JIS':
+                        self.charset = 'CP932' # XXX
                     break
         for filename in filelist:
             path = os.path.join(self.misaka_dir, filename)
-            try:
-                f = open(path)
-            except IOError:
-                if self.debug & 4:
-                    print 'cannot read', filename
-                continue
             basename, ext = os.path.splitext(filename)
-            if ext == '.__1':
-                f = StringIO.StringIO(self.crypt(f.read()))
+            try:
+                if ext == b'.__1': # should read lines as bytes
+                    f = open(path, 'rb')
+                else:
+                    f = open(path, encoding=self.charset)
+            except IOError:
+                logging.debug('cannot read {0}'.format(filename))
+                continue
+            if ext == b'.__1':
+                f = io.StringIO(self.crypt(f.read()))
             try:
                 variables, constants = self.read(f, path)
-            except MisakaError, error:
-                if self.debug & 4:
-                    print error
+            except MisakaError as error:
+                logging.debug(error)
                 continue
             global_variables.extend(variables)
             global_constants.extend(constants)
@@ -1089,7 +1085,7 @@ class Shiori:
         return 1
 
     def crypt(self, data):
-        return ''.join(map(lambda c: chr(ord(c) ^ 0xff), data))
+        return str(b''.join([int.to_bytes(c ^ 0xff, 1, 'little') for c in data]), self.charset)
 
     def eval_globals(self, sentences, constant):
         for sentence in sentences:
@@ -1100,7 +1096,7 @@ class Shiori:
                     self.eval_function(node[1], node[2])
 
     def read(self, f, path=None):
-        parser = Parser(self.charset, self.debug)
+        parser = Parser(self.charset)
         parser.read(f, path)
         common, dic = parser.get_dict()
         variables = []
@@ -1146,63 +1142,34 @@ class Shiori:
         try:
             with open(self.dbpath) as f:
                 while 1:
-                    line = f.readline()
+                    try:
+                        line = f.readline()
+                    except UnicodeError:
+                        logging.debug(
+                            'misaka.py: malformed database (ignored)')
+                        break
                     if not line:
                         break
                     header = self.strip_newline(line).split()
                     if header[0] == 'SCHOLAR':
-                        try:
-                            name = unicode(header[1], 'utf-8')
-                        except UnicodeError:
-                            try:
-                                name = unicode(header[1], 'EUC-JP')
-                            except UnicodeError:
-                                if self.debug & 4:
-                                    print 'misaka.py: malformed database (ignored)'
-                                break
+                        name = header[1]
                         value = self.strip_newline(f.readline())
-                        try:
-                            value = unicode(value, 'utf-8')
-                        except UnicodeError:
-                            try:
-                                value = unicode(value, 'EUC-JP')
-                            except UnicodeError:
-                                if self.debug & 4:
-                                    print 'misaka.py: malformed database (ignored)'
-                                break
                         self.variable[name] = (TYPE_SCHOLAR, value)
                     elif header[0] == 'ARRAY':
                         try:
                             size = int(header[1])
                         except ValueError:
-                            if self.debug & 4:
-                                print 'misaka.py: malformed database (ignored)'
+                            logging.debug(
+                                'misaka.py: malformed database (ignored)')
                             break
-                        try:
-                            name = unicode(header[2], 'utf-8')
-                        except UnicodeError:
-                            try:
-                                name = unicode(header[2], 'EUC-JP')
-                            except UnicodeError:
-                                if self.debug & 4:
-                                    print 'misaka.py: malformed database (ignored)'
-                                break
+                        name = header[2]
                         array = Array(self, name)
                         for _ in range(size):
                             value = self.strip_newline(f.readline())
-                            try:
-                                value = unicode(value, 'utf-8')
-                            except UnicodeError:
-                                try:
-                                    value = unicode(value, 'EUC-JP')
-                                except UnicodeError:
-                                    if self.debug & 4:
-                                        print 'misaka.py: malformed database (ignored)'
-                                    break
                             array.append(value)
                         self.variable[name] = (TYPE_ARRAY, array)
                     else:
-                        raise RuntimeError, 'should not reach here'
+                        raise RuntimeError('should not reach here')
         except IOError:
             return
 
@@ -1214,19 +1181,15 @@ class Shiori:
                     if value_type == TYPE_SCHOLAR:
                         if value == '':
                             continue
-                        f.write(
-                            'SCHOLAR %s\n%s\n' % \
-                                (name.encode('utf-8'), unicode(value).encode('utf-8')))
+                        f.write('SCHOLAR {0}\n{1}\n'.format(name, value))
                     elif value_type == TYPE_ARRAY:
-                        f.write(
-                            'ARRAY %d %s\n' % (len(value), name.encode('utf-8')))
+                        f.write('ARRAY {0:d} {1}\n'.format(len(value), name))
                         for item in value:
-                            f.write('%s\n' % self.expand(item).encode('utf-8'))
+                            f.write('{0}\n'.format(self.expand(item)))
                     else:
-                        raise RuntimeError, 'should not reach here'
+                        raise RuntimeError('should not reach here')
         except IOError:
-            if self.debug & 4:
-                print 'misaka.py: cannot write database (ignored)'
+            logging.debug('misaka.py: cannot write database (ignored)')
             return
 
     def unload(self):
@@ -1244,13 +1207,13 @@ class Shiori:
         header = req_string.splitlines()
         line = header.pop(0)
         if line:
-            line = line.strip()
+            line = str(line, self.charset).strip()
             req_list = line.split()
             if len(req_list) >= 2:
                 self.req_command = req_list[0].strip()
                 self.req_protocol = req_list[1].strip()
             for line in header:
-                line = line.strip()
+                line = str(line, self.charset).strip()
                 if not line:
                     continue
                 if ':' not in line:
@@ -1260,7 +1223,7 @@ class Shiori:
                 try:
                     value = int(value.strip())
                 except:
-                    value = unicode(str(value), self.charset, 'ignore').strip()
+                    value = value.strip()
                 self.req_key.append(key)
                 self.req_header[key] = value
         # FIXME
@@ -1312,16 +1275,22 @@ class Shiori:
         else:
             script = ''
         self.reset_request()
-        response = 'SHIORI/3.0 200 OK\r\n' \
-                   'Sender: Misaka\r\n' \
-                   'Value: %s\r\n' % script.encode(self.charset, 'ignore')
+        response = b''.join((b'SHIORI/3.0 200 OK\r\n',
+                             b'Sender: Misaka\r\n',
+                             b'Charset: ',
+                             self.charset.encode(self.charset, 'ignore'),
+                             b'\r\n',
+                             b'Value: ',
+                             script.encode(self.charset, 'ignore'),
+                             b'\r\n'))
         to = self.eval_variable([[NODE_TEXT, '$to']])
         if to:
             del self.variable['$to']
-            response = ''.join((response,
-                                'Reference0: %s\r\n' % \
-                                to.encode(self.charset, 'ignore')))
-        response = ''.join((response, '\r\n'))
+            response = b''.join((response,
+                                 b'Reference0: ',
+                                 to.encode(self.charset, 'ignore'),
+                                 b'\r\n'))
+        response = b''.join((response, b'\r\n'))
         return response
 
     def get_ref(self, num):
@@ -1390,8 +1359,8 @@ class Shiori:
         elif node[0] == NODE_POW_EXPR:
             result = self.eval_pow_expr(node[1:])
         else:
-            ##print node
-            raise RuntimeError, 'should not reach here'
+            ##print(node)
+            raise RuntimeError('should not reach here')
         return result
 
     SYSTEM_VARIABLES = [
@@ -1461,13 +1430,13 @@ class Shiori:
             if value_type == TYPE_ARRAY:
                 result = self.expand(value.get())
             else:
-                result = unicode(value)
+                result = str(value)
         elif name in self.variable:
             value_type, value = self.variable[name]
             if value_type == TYPE_ARRAY:
                 result = self.expand(value.get())
             else:
-                result = unicode(value)
+                result = str(value)
         else:
             result = ''
         return result
@@ -1524,7 +1493,7 @@ class Shiori:
             elif operator == '*=':
                 value *= operand
             elif operator == '/=' and operand != 0:
-                value /= operand
+                value //= operand
         if constant or name in self.constant:
             self.constant[name] = (TYPE_SCHOLAR, value)
         else:
@@ -1598,7 +1567,7 @@ class Shiori:
             if expression[i] == '*':
                 value *= operand
             elif expression[i] == '/' and operand != 0:
-                value /= operand
+                value //= operand
             elif expression[i] == '%' and operand != 0:
                 value %= operand
         return str(value)
@@ -1635,7 +1604,7 @@ class Shiori:
         return function(self, args)
 
     def is_number(self, s):
-        return s and filter(lambda c: c in '0123456789', s) == s
+        return s and all([bool(c in '0123456789') for c in s])
 
     def split(self, s):
         buf = []
@@ -1656,7 +1625,7 @@ class Shiori:
         if n in '01234567':
             value = self.reference[int(n)]
             if value is not None:
-                return unicode(value)
+                return str(value)
         return ''
 
     def exec_random(self, args):
@@ -1708,7 +1677,7 @@ class Shiori:
         return self.eval_variable([[NODE_TEXT, random.choice(keylist)]])
 
     def keys(self):
-        buf = self.dict.keys()
+        buf = list(self.dict.keys())
         buf.extend(self.variable.keys())
         buf.extend(self.constant.keys())
         return buf
@@ -1760,7 +1729,7 @@ class Shiori:
         if offset < 0:
             return ''
         s = self.expand_args(args[0]).encode(self.charset, 'replace')
-        return unicode(s[offset:offset + count], self.charset, 'replace')
+        return str(s[offset:offset + count], self.charset, 'replace')
 
     def exec_substringl(self, args):
         if len(args) != 2:
@@ -1773,7 +1742,7 @@ class Shiori:
         if count < 0:
             return ''
         s = self.expand_args(args[0]).encode(self.charset, 'replace')
-        return unicode(s[:count], self.charset, 'replace')
+        return str(s[:count], self.charset, 'replace')
 
     def exec_substringr(self, args):
         if len(args) != 2:
@@ -1786,7 +1755,7 @@ class Shiori:
         if count < 0:
             return ''
         s = self.expand_args(args[0]).encode(self.charset, 'replace')
-        return unicode(s[len(s) - count:], self.charset, 'replace')
+        return str(s[len(s) - count:], self.charset, 'replace')
 
     def exec_substringfirst(self, args):
         if len(args) != 1:
@@ -1830,12 +1799,10 @@ class Shiori:
         if len(args) != 1:
             return ''
         buf = []
-        string = unicode(self.expand_args(args[0]), self.charset, 'replace')
-        #string = unicode(args[0], self.charset, 'replace')
+        string = str(self.expand_args(args[0]), self.charset, 'replace')
+        #string = str(args[0], self.charset, 'replace')
         for c in string:
-            c = c.encode('utf-8')
             c = self.katakana2hiragana.get(c, c)
-            c = unicode(c, 'utf-8')
             buf.append(c)
         return ''.join(buf)
 
@@ -1986,7 +1953,6 @@ class Shiori:
             return ''
         s = self.expand_args(args[0])
         for prefix in self.prefixes:
-            prefix = unicode(prefix, 'utf-8')
             if s.endswith(prefix):
                 s = ''.join((s[:-len(prefix)], self.expand_args(args[1])))
                 break
@@ -2018,26 +1984,28 @@ class Shiori:
         saori_header = []
         saori_value = {}
         saori_protocol = ''
-        req = 'EXECUTE SAORI/1.0\r\n' \
-              'Sender: MISAKA\r\n' \
-              'SecurityLevel: local\r\n' \
-              'Charset: %s\r\n' % self.charset
+        req = b''.join((b'EXECUTE SAORI/1.0\r\n',
+                        b'Sender: MISAKA\r\n',
+                        b'SecurityLevel: local\r\n',
+                        b'Charset: ',
+                        self.charset.encode(self.charset),
+                        b'\r\n'))
         for i in range(1, len(args)):
-            req = ''.join(
+            req = b''.join(
                 (req,
-                 'Argument%s: %s\r\n' % \
-                 (i,
-                  self.expand_args(args[i]).encode(self.charset, 'ignore'))))
-        req = ''.join((req, '\r\n'))
+                 b'Argument', str(i).encode(self.charset), b': ', 
+                 self.expand_args(args[i]).encode(self.charset, 'ignore'),
+                 b'\r\n'))
+        req = b''.join((req, b'\r\n'))
         response = self.saori_library.request(self.expand_args(args[0]), req)
         header = response.splitlines()
-        line = header.pop(0)
-        if line:
-            line = line.strip()
+        if header:
+            line = header.pop(0)
+            line = str(line, self.charset).strip()
             if ' ' in line:
                 saori_protocol, saori_statuscode = [x.strip for x in line.split(' ', 1)]
             for line in header:
-                line = line.strip()
+                line = str(line, self.charset).strip()
                 if not line:
                     continue
                 if ':' not in line:
@@ -2111,6 +2079,7 @@ class Shiori:
         '$unloadsaori':         unload_saori,
         }
 
+
 class SaoriLibrary:
 
     def __init__(self, saori, top_dir):
@@ -2119,7 +2088,8 @@ class SaoriLibrary:
 
     def load(self, name, top_dir):
         result = 0
-        name = os.path.split(name.replace('\\', '/'))[-1] # XXX
+        head, name = os.path.split(name.replace('\\', '/')) # XXX: don't use os.fsencode() here
+        top_dir = os.path.join(top_dir, os.fsencode(head))
         if name not in self.saori_list:
             module = self.saori.request(name)
             if module:
@@ -2130,25 +2100,25 @@ class SaoriLibrary:
 
     def unload(self, name=None):
         if name:
-            name = os.path.split(name.replace('\\', '/'))[-1] # XXX
+            name = os.path.split(name.replace('\\', '/'))[-1] # XXX: don't use os.fsencode() here
             if name in self.saori_list:
                 self.saori_list[name].unload()
                 del self.saori_list[name]
         else:
-            for saori in self.saori_list.itervalues():
+            for saori in self.saori_list.values():
                 saori.unload()
         return None
 
     def request(self, name, req):
         result = '' # FIXME
-        name = os.path.split(name.replace('\\', '/'))[-1] # XXX
+        name = os.path.split(name.replace('\\', '/'))[-1] # XXX: don't use os.fsencode() here
         if name and name in self.saori_list:
             result = self.saori_list[name].request(req)
         return result
 
 
-def misaka_open(top_dir, debug=0):
-    misaka = Shiori(top_dir, debug)
+def misaka_open(top_dir):
+    misaka = Shiori(top_dir)
     misaka.load()
     return misaka
 
@@ -2156,19 +2126,21 @@ def misaka_open(top_dir, debug=0):
 
 def test_ini(dirlist):
     for top_dir in dirlist:
-        print 'Reading', os.path.join(top_dir, 'misaka.ini'), '...'
-        filelist, debug, error = read_misaka_ini(top_dir, 4)
-        print 'number of dictionaries =', len(filelist)
+        top_dir = os.fsencode(top_dir)
+        print('Reading', os.path.join(top_dir, b'misaka.ini'), '...')
+        filelist, debug, error = read_misaka_ini(top_dir)
+        print('number of dictionaries =', len(filelist))
         for filename in filelist:
-            print filename
-        print 'debug =', debug
-        print 'error =', error
+            print(filename)
+        print('debug =', debug)
+        print('error =', error)
 
-def test_lexer(filelist, charset='Shift_JIS'):
-    lexer = Lexer(charset=charset, debug=1)
+def test_lexer(filelist, charset='CP932'):
+    lexer = Lexer(charset=charset)
     for filename in filelist:
-        print 'Reading', filename, '...'
-        lexer.read(open(filename))
+        filename = os.fsencode(filename)
+        print('Reading', filename, '...')
+        lexer.read(open(filename, encoding=charset))
     token_names = {
         TOKEN_WHITESPACE:    'TOKEN_WHITESPACE',
         TOKEN_NEWLINE:       'TOKEN_NEWLINE',
@@ -2186,68 +2158,71 @@ def test_lexer(filelist, charset='Shift_JIS'):
         TOKEN_TEXT:          'TOKEN_TEXT',
         }
     for token, lexeme, position in lexer.buffer:
-        print 'L%d : C%-3d :' % position, token_names[token], ':',
+        print('L{0:d} : C{1:-3d} :'.format(*position), token_names[token], ':',)
         if token in [TOKEN_WHITESPACE, TOKEN_NEWLINE,
                      TOKEN_OPEN_BRACE, TOKEN_CLOSE_BRACE,
                      TOKEN_OPEN_PAREN, TOKEN_CLOSE_PAREN,
                      TOKEN_OPEN_BRACKET, TOKEN_CLOSE_BRACKET,
                      TOKEN_DOLLAR,
                      TOKEN_COMMA, TOKEN_SEMICOLON]:
-            print repr(lexeme)
+            print(repr(lexeme))
         else:
-            print lexeme.encode('utf-8', 'ignore')
+            print(lexeme)
 
-def test_parser(filelist, charset='Shift_JIS'):
+def test_parser(filelist, charset='CP932'):
     for filename in filelist:
-        print 'Reading', filename, '...'
-        parser = Parser(charset=charset, debug=1)
-        parser.read(open(filename))
+        filename = os.fsencode(filename)
+        print('Reading', filename, '...')
+        parser = Parser(charset=charset)
+        parser.read(open(filename, encoding=charset))
         common, dic = parser.get_dict()
         if common is not None:
-            print 'Common'
+            print('Common')
             parser.dump_node(common, depth=2)
         for name, parameters, sentences in dic:
-            print 'Group', name.encode('utf-8', 'ignore')
+            print('Group', name)
             if parameters:
-                print 'Parameter:'
+                print('Parameter:')
                 for parameter in parameters:
-                    print '>>>', parameter
+                    print('>>>', parameter)
                     parser.dump_node(parameter, depth=2)
             if sentences:
-                print 'Sentence:'
+                print('Sentence:')
                 for sentence in sentences:
-                    print '>>>', sentence
+                    print('>>>', sentence)
                     parser.dump_list(sentence, depth=2)
 
 def test_interpreter(top_dir):
-    misaka = Shiori(top_dir, debug=1)
-    misaka.load()
+    misaka = Shiori('misaka.dll') # XXX
+    misaka.load(os.fsencode(top_dir))
     while 1:
         try:
-            line = raw_input('>>> ')
+            line = input('>>> ')
         except KeyboardInterrupt:
-            print 'Break'
+            print('Break')
             continue
         except EOFError:
-            print
+            print()
             break
         command = line.split()
         if not command:
             continue
         elif len(command) == 1 and command[0].startswith('$'):
-            print misaka.eval_variable([[NODE_TEXT, command[0]]])
+            print(misaka.eval_variable([[NODE_TEXT, command[0]]]))
         elif len(command) == 1 and command[0] == 'reload':
             misaka.load()
         else:
-            print 'list of commands:'
-            print '  $id     get variable value'
-            print '  reload  reload dictionaries'
+            print('list of commands:')
+            print('  $id     get variable value')
+            print('  reload  reload dictionaries')
 
 if __name__ == '__main__':
+    logger = logging.getLogger()
+    logger.setLevel(logging.DEBUG) # XXX
     USAGE= 'Usage: misaka.py [ini|lexer|parser|interp] ...'
-    charset = 'Shift_JIS' # XXX
+    charset = 'CP932' # XXX
     if len(sys.argv) == 1:
-        print USAGE
+        print(USAGE)
     elif sys.argv[1] == 'ini':
         test_ini(sys.argv[2:])
     elif sys.argv[1] == 'lexer':
@@ -2257,4 +2232,4 @@ if __name__ == '__main__':
     elif sys.argv[1] == 'interp':
         test_interpreter(sys.argv[2])
     else:
-        print USAGE
+        print(USAGE)
