@@ -121,6 +121,7 @@ class Sakura:
         self.anchor = None
         self.clock = (0, 0)
         self.synchronized_session = []
+        self.force_quit = False
         ##
         self.old_otherghostname = None ## FIXME
         # create vanish dialog
@@ -822,20 +823,62 @@ class Sakura:
             return
         if self.time_critical_session:
             return
-        elif button in [1, 3] and click == 1:
+        elif click == 1:
             if self.passivemode and \
                self.processed_script is not None:
                 return
             part = self.surface.get_touched_region(side, x, y)
-            self.notify_event('OnMouseClick', x, y, 0, side, part, button)
+            if button in {1, 2, 3}:
+                num_button = [0, 2, 1][button - 1]
+                if not self.notify_event('OnMouseUp',
+                                         x, y, 0, side, part, num_button,
+                                         'mouse'): # FIXME
+                    if button == 2:
+                        if self.notify_event(
+                            'OnMouseUpEx',
+                            x, y, 0, side, part, 'middle',
+                            'mouse'): # FIXME
+                            return
+                        if self.notify_event('OnMouseClickEx',
+                                      x, y, 0, side, part, 'middle',
+                                      'mouse'): # FIXME
+                            return
+                    self.notify_event('OnMouseClick',
+                                      x, y, 0, side, part, num_button,
+                                      'mouse') # FIXME
+            elif button in {8, 9}:
+                ex_button = {
+                    2: 'middle',
+                    8: 'xbutton1',
+                    9: 'xbutton2'
+                    }[button]
+                if not self.notify_event('OnMouseUpEx',
+                                         x, y, 0, side, part, ex_button,
+                                         'mouse'): # FIXME
+                    self.notify_event('OnMouseClickEx',
+                                      x, y, 0, side, part, ex_button,
+                                      'mouse') # FIXME
         elif self.passivemode:
             return
-        elif button == 1 and click == 2:
+        elif button in {1, 3} and click == 2:
             if self.sstp_request_handler:
                 self.sstp_request_handler.send_sstp_break()
                 self.sstp_request_handler = None
             part = self.surface.get_touched_region(side, x, y)
-            self.notify_event('OnMouseDoubleClick', x, y, '', side, part)
+            num_button = [0, 2, 1][button - 1]
+            self.notify_event('OnMouseDoubleClick',
+                              x, y, 0, side, part, num_button,
+                              'mouse') # FIXME
+        elif button in {2, 8, 9} and click == 2:
+            part = self.surface.get_touched_region(side, x, y)
+            ex_button = {
+                2: 'middle',
+                8: 'xbutton1',
+                9: 'xbutton2'
+                }[button]
+            self.notify_event('OnMouseDoubleClickEx',
+                              x, y, 0, side, part, ex_button,
+                              'mouse') # FIXME
 
     def notify_balloon_click(self, button, click, side):
         if self.script_mode == self.PAUSE_MODE:
@@ -923,10 +966,22 @@ class Sakura:
                         else str(value)
                     logging.debug(
                     'Reference{0:d}: {1}'.format(n, value))
+        if event == 'OnCloseAll':
+            self.force_quit = True
+            if not script: # fallback
+                script, communication = \
+                    self.get_event_response_with_communication(
+                    'OnClose', *arglist, **argdict) or (default, None)
+            if script:
+                self.start_script(script)
+                self.balloon.hide_sstp_message()
+            return 1
+        if event == 'OnClose' and arglist[0] == 'shutdown': # XXX
+            self.force_quit = True
         if not script: # an empty script is ignored
             if event in self.boot_event:
                 self.surface_bootup()
-            if event == 'OnMouseClick' and arglist[5] == 3:
+            if event == 'OnMouseClick' and arglist[5] == 1:
                 self.request_parent(
                     'NOTIFY', 'open_popup_menu', self, arglist[5], arglist[3])
             self.request_parent(
@@ -1240,6 +1295,7 @@ class Sakura:
         self.__running = 1
         self.__temp_mode = temp
         self.key = key
+        self.force_quit = False
         logging.info('ghost {0}'.format(key))
         self.load_settings()
         shell_key = self.get_default_shell()
@@ -1289,6 +1345,7 @@ class Sakura:
         self.position_all()
         self.reset_idle_time()
         self.__running = 1
+        self.force_quit = False
 
     def stop(self):
         if not self.__running:
@@ -1426,6 +1483,9 @@ class Sakura:
     def do_idle_tasks(self):
         if not self.__running:
             return False
+        if self.force_quit and not self.busy() and \
+           not (self.processed_script or self.processed_text):
+            self.quit()
         self.request_parent('NOTIFY', 'update_working', self.get_name())
         if self.__temp_mode:
             self.process_script()
@@ -1987,7 +2047,7 @@ class Sakura:
                 self.__listening['OnOtherSurfaceChange'] = False
             else:
                 pass ## FIXME
-        elif args[0:1] == 'sound' and argc > 1:
+        elif args[0] == 'sound' and argc > 1:
             command = args[1]
             if self.audio_player is None:
                 return
@@ -2039,15 +2099,38 @@ class Sakura:
                     self.audio_player.set_state(Gst.State.PLAYING)
             else:
                 pass ## FIXME
-        elif args[0:1] == '*':
+        elif args[0] == '*':
             self.balloon.append_sstp_marker(self.script_side)
-        elif args[0:1] == 'quicksession' and argc > 1:
+        elif args[0] == 'quicksession' and argc > 1:
             if args[1] == 'true':
                 self.quick_session = 1
             elif args[1] == 'false':
                 self.quick_session = 0
             else:
                 pass ## FIXME
+        elif args[0] == 'bind' and argc > 2:
+            category = args[1]
+            name = args[2]
+            if argc < 4:
+                flag = 'toggle'
+            else:
+                flag = args[3]
+            bind = self.surface.window[self.script_side].bind # XXX
+            for key in bind:
+                group = bind[key][0].split(',')
+                if category != group[0]:
+                    continue
+                if name and name != group[1]:
+                    continue
+                if flag in ['true', '1']:
+                    if bind[key][1]:
+                        continue
+                elif flag in ['false', '0']:
+                    if not bind[key][1]:
+                        continue
+                else: # 'toggle'
+                    pass
+                self.surface.toggle_bind((self.script_side, key))
         else:
             pass ## FIXME
 
