@@ -700,6 +700,7 @@ class SurfaceWindow:
         self.dragged = False
         self.x_root = None
         self.y_root = None
+        self.click_count = 0
         self.window.connect('leave_notify_event', self.window_leave_notify) # XXX
         self.window.connect('enter_notify_event', self.window_enter_notify) # XXX
         # create drawing area
@@ -1175,27 +1176,48 @@ class SurfaceWindow:
         y = int(y * 100 / scale)
         self.x_root = event.x_root
         self.y_root = event.y_root
-        click = 1 if event.type == Gdk.EventType.BUTTON_PRESS else 2
         # automagical raise
         self.request_parent('NOTIFY', 'notify_observer', 'raise', (self.side))
-        self.request_parent('NOTIFY', 'notify_surface_click',
-                            event.button, click, self.side, x, y)
+        delivered, click_count = event.get_click_count()
+        if delivered:
+            self.click_count = click_count
+            if event.button in {1, 2, 3}:
+                num_button = [0, 2, 1][event.button - 1]
+                self.request_parent('NOTIFY', 'notify_event', 'OnMouseDown',
+                                    x, y, 0, self.side, self.__current_part,
+                                    num_button,
+                                    'mouse') # FIXME
+            if event.button in {2, 8, 9}:
+                ex_button = {
+                    2: 'middle',
+                    8: 'xbutton1',
+                    9: 'xbutton2'
+                    }[event.button]
+                self.request_parent('NOTIFY', 'notify_event', 'OnMouseDownEx',
+                                    x, y, 0, self.side, self.__current_part,
+                                    ex_button,
+                                    'mouse') # FIXME
         return True
 
     def button_release(self, window, event):
+        x = int(event.x)
+        y = int(event.y)
+        scale = self.scale
+        x = int(x * 100 / scale)
+        y = int(y * 100 / scale)
         if self.dragged:
             self.dragged = False
             self.set_alignment_current()
-            x = int(event.x)
-            y = int(event.y)
-            scale = self.scale
-            x = int(x * 100 / scale)
-            y = int(y * 100 / scale)
             self.request_parent(
                 'NOTIFY', 'notify_event',
                 'OnMouseDragEnd', x, y, '', self.side, self.__current_part, '')
         self.x_root = None
         self.y_root = None
+        if self.click_count > 0:
+            self.request_parent('NOTIFY', 'notify_surface_click',
+                                event.button, self.click_count,
+                                self.side, x, y)
+            self.click_count = 0
         return True
 
     def motion_notify(self, darea, event):
@@ -1273,10 +1295,12 @@ class SurfaceWindow:
         if bind_id in self.bind:
             current = self.bind[bind_id][1]
             self.bind[bind_id][1] = not current
+            group = self.bind[bind_id][0].split(',')
             self.request_parent('NOTIFY', 'notify_event',
                                 'OnDressupChanged', self.side,
-                                self.bind[bind_id][0],
-                                1 if self.bind[bind_id][1] else 0)
+                                group[1],
+                                1 if self.bind[bind_id][1] else 0,
+                                group[0])
             self.reset_surface()
 
     def window_enter_notify(self, window, event):
